/*   mosmixparser - parses MOSMIX open data from DWD to create a
 *   HTML5 based weather forecast page.
 *
 *   Copyright (C) 2021 Thomas Rudolph, <https://rudo.info/mosmix.php>
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

// Erzeuge HTML-Datei. Durch die Einkapselung der HTML- und Javascript-Struktur ergibt sich ziemlicher Spaghetticode.
int html(char* argv[], uint8_t hPos) {

    // Bevor die HTML-Ausgabe erfolgt, werden die DWD-Werte für jeweils 6 Stunden gemittelt, um sie sinnvoll als HTML darstellen zu können.
    float t_ttt[41];
    float t_n[41];
    float t_rr1[41];
    float t_r101[41];
    float t_wwM6[41];
    float t_RRS1c[41];
    float t_FF[41];
    float t_DD[41];
    float t_FX1[41];
    float t_SunD1[41];
    float t_wwT[41];
    uint8_t zeitpunkt;

    // Es bleiben dann für die zehn Tage Vorhersage 41 Messwerte übrig, durch die wir nun iterieren und den Mittelwert berechnen.
    for (uint8_t i=0; i< 41; i++)
    {
        t_ttt[i] = t_n[i] = t_rr1[i] = t_r101[i] = t_wwM6[i] = t_RRS1c[i] = t_FF[i] = t_DD[i] = t_FX1[i] = t_wwT[i] = t_SunD1[i] = 0;
        try {
            zeitpunkt = stoi(timestamp[6*i].substr(11,2));
        }
        catch( ... )
        {
             cout << "Es wurden ungültige Zeitstempel eingelesen. Möglicherweise ist die MOSMIX-Datei\nunvollständig oder für die Station existieren einzelne Werte nicht.\n";
             exit(0);
        }

        for (uint8_t j=0; j< 6; j++)
        {
            // Bei der Temperatur wird kein Mittelwert gebildet wie bei den anderen Variablen. Tagsüber wird stattdessen die Höchst-, nachts die Tiefsttemperatur berechnet.
            if (zeitpunkt < 8 || zeitpunkt > 18)
            {
                if (j == 0) t_ttt[i] = ttt[6*i + j];
                else if (t_ttt[i] > ttt[6*i + j]) t_ttt[i] = ttt[6*i + j];
            }
            else
            {
                if (j == 0) t_ttt[i] = ttt[6*i + j];
                else if (t_ttt[i] < ttt[6*i + j]) t_ttt[i] = ttt[6*i + j];
            }

            t_n[i] += n[6*i + j];
            t_rr1[i] += rr1[6*i + j];
            t_r101[i] += r101[6*i + j];
            t_wwM6[i] += wwM6[6*i + j];
            t_RRS1c[i] += RRS1c[6*i + j];
            t_FF[i] += FF[6*i + j];
            t_DD[i] += DD[6*i + j];
            if (FX1[6*i + j] > t_FX1[i]) t_FX1[i] = FX1[6*i + j];
            t_SunD1[i] += SunD1[6*i + j];
            if (wwT[6*i + j] > t_wwT[i]) t_wwT[i] = wwT[6*i + j];
        }

        t_n[i] /= 6;
        t_r101[i] /= 6;
        t_wwM6[i] /= 6;
        t_RRS1c[i] /= 6;
        t_FF[i] /= 6;
        t_DD[i] /= 6;
        t_SunD1[i] /= 216;
    }

    int tempTag, tempMon, tempJahr, tempUhr, altTag, wtag, prefix=0;
    try {
        // Holen des Datums zu Beginns der Vorschauwerte.
        tempTag = stoi(timestamp[0].substr(8,2));
        tempMon = stoi(timestamp[0].substr(5,2));
        tempJahr = stoi(timestamp[0].substr(0,4));

        // Der Präfix wird benötigt, um die Graphen unten auf der Seite mit der Datumsanzeige zu synchronisieren.
        if (stoi(timestamp[16].substr(8,2)) == tempTag) prefix = 3;
        else if (stoi(timestamp[12].substr(8,2)) == tempTag) prefix = 4;
        else if (stoi(timestamp[10].substr(8,2)) == tempTag) prefix = 2;
        else if (stoi(timestamp[5].substr(8,2)) == tempTag) prefix = 1;
    }
    catch( ... )
    {
         cout << "Es wurden ungültige Zeitstempel eingelesen. Möglicherweise ist die MOSMIX-Datei\nunvollständig oder für die Station existieren einzelne Werte nicht.\n";
         exit(0);
    }

    ofstream html(argv[hPos], ios::out);

    if (!html.is_open())
    {
        cout << "HTML-Datei konnte nicht geschrieben werden.\n";
        exit(0);
    }

    // Schreibe den HTML-Kopf inklusive Javascript für die Graphen.
    html << "<html><head><meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\"><title>Wetterfrosch</title>\n";
    html << "<link rel=\"icon\" href=\"data:image/svg+xml,<svg xmlns=%22http://www.w3.org/2000/svg%22 viewBox=%220 0 100 100%22><text y=%22.9em%22 font-size=%2290%22>🐸️</text></svg>\">\n";
    html << "<style type=\"text/css\">#tage { background:linear-gradient(#f9f9ff,#ffffff); display:inline-block; font-size:12pt; align:center; border:0px solid rgba(118,143,240,.5);border-right:1px solid rgba(78,113,200,.5); border-bottom:1px solid rgba(48,73,130,.5); margin:4px; padding:18px; width:189px; text-align:center;}\n";
    html << "body {font-family:DejaVu Sans,Verdana,Tahoma,Georgia,Roboto,Helvetica,Arial;}\n</style>\n";

    html << "<script type=\"text/javascript\">var canvas; var context; var Val_max; var Val_min; var sections; var xScale; var yScale;\n var Temp =  [";
    // Schreibe die Messwerte in die Javascript-Variablen für die Graphen.
    for (uint8_t i=0; i<241; i++)
    {
        html << "\"" << setprecision(2) << ttt[i] << "\", ";
    }

    html << "]; var SunD1 = [";
    for (uint8_t i=0; i<241; i++)
    {
        html << "\"" << setprecision(2) << SunD1[i]/36 << "\", ";
    }

    html << "]; var rr1 = [";
    for (uint8_t i=0; i<241; i++)
    {
        html << "\"" << setprecision(2) << r101[i] << "\", ";
    }

    html << "]; var FF = [";
    for (uint8_t i=0; i<241; i++)
    {
        html << "\"" << setprecision(2) << FF[i]*3.6 << "\", ";
    }

    html << "];\n function init() {\n sections = 241; Val_max = Math.max(...Temp); Val_min = Math.min(...Temp); var stepSize = 2; var columnSize = 50; var rowSize = 50; var margin = 10;\n";
    html << "var xAxis = [";

    // Schreibe Präfix in die x-Achse, um den Graph mit der Tageszeit zu synchronisieren. Freilich keine allzu elegante Lösung ...
    if (prefix == 3) html << "\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"";
    if (prefix == 2) html << "\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"";
    if (prefix == 4) html << "\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"";
    if (prefix == 1) html << "\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"";
    if (prefix == 0) html << "\"\",\"\",\"\",\"";

    // Berechne Datum in der x-Achse modulo Monats- und Jahresende. Zumindest bis ins Jahr 2100.
    for (uint8_t i=0; i<241; i+=24)
    {
        tempTag++;
        if (tempTag > 30 && (tempMon == 4 || tempMon == 6 || tempMon == 9 || tempMon == 11))
        {
            tempTag = 1;
        }
        if (tempTag == 29 && tempMon == 2)
        {
            if ((tempJahr % 4) != 0)
            {
                tempTag = 1;
            }
        }
        if (tempTag == 30 && tempMon == 2)
        {
            tempTag = 1;
        }
        if (tempTag > 31)
        {
            tempTag = 1;
        }

        html << "| " << "\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"" << tempTag << ".\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"\",\"";
    }

    // Restlicher Javascript-Spaß; erster Graph (Temperatur).
    html << "\"];\n canvas = document.getElementById(\"canvas\"); context = canvas.getContext(\"2d\"); canvas.width=(window.innerWidth)*0.7; context.fillStyle = \"#002288\"; context.font = \"20 pt Verdana\";\n";
    html << "yScale = (canvas.height - columnSize - margin) / (Val_max - Val_min); xScale = (canvas.width - rowSize) / sections; context.strokeStyle=\"#003333\"; context.beginPath(); for (i=1;i<=sections;i++) { var x = i * xScale; context.fillText(xAxis[i], x+45,columnSize - margin);}\n";
    html << "var count =  0; for (scale=Val_max;scale>=Val_min;scale = scale - stepSize) { var y = columnSize + (yScale * count * stepSize); context.fillText(scale.toFixed(1), margin,y + margin); context.moveTo(rowSize,y); context.lineTo(canvas.width,y); count++;}\n";

    html << "context.stroke(); context.translate(rowSize,canvas.height + Val_min * yScale); context.scale(1,-1 * yScale);";
    html << "context.strokeStyle=\"#DD0033\"; plotData(Temp);\n";

    // zweiter Graph (Sonnenscheindauer).
    html << "stepSize = 5; Val_max = 100; Val_min = 0; canvas = document.getElementById(\"sonne\"); context = canvas.getContext(\"2d\"); canvas.width=(window.innerWidth)*0.7;\nyScale = (canvas.height - columnSize - margin) / (Val_max - Val_min); xScale = (canvas.width - rowSize) / sections;\n";
    html << "for (i=1;i<=sections;i++) { var x = i * xScale; context.fillText(xAxis[i], x+45,columnSize - margin);}\n";
    html << "var count = 0; for (scale=Val_max;scale>=Val_min;scale = scale - stepSize) { var y = columnSize + (yScale * count * stepSize); context.fillText(scale, margin,y + margin); context.moveTo(rowSize,y); context.lineTo(canvas.width,y); count++;}\n";
    html << "context.stroke(); context.translate(rowSize,canvas.height + Val_min * yScale); context.scale(1,-1 * yScale);context.strokeStyle=\"#cccc00\"; plotData(SunD1);\n";

    // dritter Graph (Regen).
    html << " Val_max = 100; Val_min = 0; canvas = document.getElementById(\"regen\"); context = canvas.getContext(\"2d\"); canvas.width=(window.innerWidth)*0.7;\nyScale = (canvas.height - columnSize - margin) / (Val_max - Val_min); xScale = (canvas.width - rowSize) / sections;\n";
    html << "for (i=1;i<=sections;i++) { var x = i * xScale; context.fillText(xAxis[i], x+45,columnSize - margin);}\n";
    html << "var count = 0; for (scale=Val_max;scale>=Val_min;scale = scale - stepSize) { var y = columnSize + (yScale * count * stepSize); context.fillText(scale, margin,y + margin); context.moveTo(rowSize,y); context.lineTo(canvas.width,y); count++;}\n";
    html << "context.stroke(); context.translate(rowSize,canvas.height + Val_min * yScale); context.scale(1,-1 * yScale);context.strokeStyle=\"#0000cc\"; plotData(rr1);\n";

    // vierter Graph (Wind).
    html << " Val_max = Math.max(...FF); Val_min = 0; canvas = document.getElementById(\"wind\"); context = canvas.getContext(\"2d\"); canvas.width=(window.innerWidth)*0.7;\nyScale = (canvas.height - columnSize - margin) / (Val_max - Val_min); xScale = (canvas.width - rowSize) / sections;\n";
    html << "for (i=1;i<=sections;i++) { var x = i * xScale; context.fillText(xAxis[i], x+45,columnSize - margin);}\n";
    html << "var count = 0; for (scale=Val_max;scale>=Val_min;scale = scale - stepSize) { var y = columnSize + (yScale * count * stepSize); context.fillText(scale, margin,y + margin); context.moveTo(rowSize,y); context.lineTo(canvas.width,y); count++;}\n";
    html << "context.stroke(); context.translate(rowSize,canvas.height + Val_min * yScale); context.scale(1,-1 * yScale);context.strokeStyle=\"#00cc00\"; plotData(FF);}\n";

    html << "function plotData(dataSet) { context.beginPath(); context.moveTo(0, dataSet[0]); for (i=1;i<sections;i++) { context.lineTo(i * xScale, dataSet[i]); } context.stroke();} </script>\n";

    // Javascript-Verkapselung fertig.


    html << "</head><body onLoad=\"init()\">\nWettervorhersage erstellt am ";

    // Holen der Systemzeit.
    time_t sekunden = time(NULL);
    tm *uhr = localtime(&sekunden);

    html << uhr->tm_year+1900 << "-";
        if ((uhr->tm_mon)+1 < 10) html << "0";
        html << (uhr->tm_mon)+1 << "-";
        if (uhr->tm_mday < 10) html << "0";
        html << uhr->tm_mday << " um " << uhr->tm_hour << ":";
    if (uhr->tm_min < 10) html << "0";
    html << uhr->tm_min << "Uhr.";
    html << "\n<br><br>";

    tempTag = 0;

    // Erzeugen der Tagesboxen mit ihren Werten in HTML für alle 41 Iterationen.
    for (uint8_t i=0; i< 41; i++)
    {
        altTag = tempTag;

        // Berechne Datum in den Boxen modulo Monats- und Jahresende. Zumindest bis ins Jahr 2100.
        tempTag = stoi(timestamp[6*i].substr(8,2));
        tempMon = stoi(timestamp[6*i].substr(5,2));
        tempUhr = stoi(timestamp[6*i].substr(11,2));

        // Prüfe, ob Datum in der Sommerzeit liegt.
        if (tempMon > 3 && tempMon < 10) tempUhr+=2;
        else if (tempMon == 3 || tempMon == 10)
        {
            // Berechnen des Wochentages.
            static int t[] = {0, 3, 2, 5, 0, 3, 5, 1, 4, 6, 2, 4};
            int jahr = tempJahr;
            jahr -= tempMon < 3;
            wtag = (jahr + jahr/4 - jahr/100 + jahr/400 + t[tempMon-1]) % 7;
            int zeitumstellung = 7 - wtag + 21;
            if (tempTag >= zeitumstellung && tempMon == 3) tempUhr+= 2;
            else if (tempTag < zeitumstellung && tempMon == 10) tempUhr+= 2;
            else tempUhr++;
        }
        else tempUhr++;
        tempJahr = stoi(timestamp[6*i].substr(0,4));

        // Zähle Werte zum nächsten Tag, wenn die lokale Zeit im nächsten Tag liegt, die UTC aber nicht.
        if (tempUhr >= 24) tempTag += 1;
        if (tempUhr == 24) tempUhr = 0;

        // Datumskorrekturen bei Monats- und Jahreswechsel.
        if (tempTag > 30 && (tempMon == 4 || tempMon == 6 || tempMon == 9 || tempMon == 11))
        {
            tempTag = 1;
            tempMon++;
        }
        if (tempTag == 29 && tempMon == 2)
        {
            if ((tempJahr % 4) != 0)
            {
                tempTag = 1;
                tempMon++;
            }
        }
        if (tempTag == 30 && tempMon == 2)
        {
            tempTag = 1;
            tempMon++;
        }
        if (tempTag > 31)
        {
            tempTag = 1;
            tempMon++;
        }
        if (tempMon == 13)
        {
            tempMon = 1;
            tempJahr++;
        }

        string monText, wtagText;

        // Berechnen des Wochentages.
        static int t[] = {0, 3, 2, 5, 0, 3, 5, 1, 4, 6, 2, 4};
        int jahr = tempJahr;
        jahr -= tempMon < 3;
        wtag = (jahr + jahr/4 - jahr/100 + jahr/400 + t[tempMon-1] + tempTag) % 7;

        if (wtag == 0) wtagText = "Sonntag";
        else if (wtag == 1) wtagText = "Montag";
        else if (wtag == 2) wtagText = "Dienstag";
        else if (wtag == 3) wtagText = "Mittwoch";
        else if (wtag == 4) wtagText = "Donnerstag";
        else if (wtag == 5) wtagText = "Freitag";
        else if (wtag == 6) wtagText = "Samstag";

        if (tempMon == 1)  monText = "Januar";
        if (tempMon == 2)  monText = "Februar";
        if (tempMon == 3)  monText = "März";
        if (tempMon == 4)  monText = "April";
        if (tempMon == 5)  monText = "Mai";
        if (tempMon == 6)  monText = "Juni";
        if (tempMon == 7)  monText = "Juli";
        if (tempMon == 8)  monText = "August";
        if (tempMon == 9)  monText = "September";
        if (tempMon == 10) monText = "Oktober";
        if (tempMon == 11) monText = "November";
        if (tempMon == 12) monText = "Dezember";

        // Hier beginnt das HTML-CSS-Spaghetti.
        if (altTag != tempTag && i > 0) html << "</div><div id=\"tage\">" << wtagText << "<br><b>" << tempTag << ". " << monText << "</b>";
        else if (i == 0) html << "<div id=\"tage\">" << wtagText << "<br><b>" << tempTag << ". " << monText << "</b>";

        // Schreibe Zeitpunkt.
        html << "<br><span style=\"color: #410;\">" << tempUhr << "&nbsp;";
        tempUhr += 5;
        if (tempUhr >= 24) tempUhr -= 24;
        html << "bis " << tempUhr << "&nbsp;Uhr</span>";

        string phase = moonPhase((uhr->tm_yday+1+i/4)%365, tempJahr);

        // Die Temperaturanzeige wird farblich angepasst. Je wärmer desto roter, je kälter desto blauer.
        int16_t tempBlue = 255-(t_ttt[i]+10)*6;
        int16_t tempRed = (t_ttt[i]+10)*6;
        if (tempBlue < 0) tempBlue = 0;
        if (tempBlue > 255) tempBlue = 255;
        if (tempRed < 0) tempRed = 0;
        if (tempRed > 255) tempRed = 255;
        int16_t tempBluen = 255-t_n[i]*0.1;

        // Die Messwerte werden in eine kleine HTML-Tabelle gepackt.
        html << setprecision(0) << "<table style=\"font-size:12pt;margin-top:5px;\"><tr style=\"height:40px;\">";

        html << "<td><b><span style=\"color:rgba(" << (int)tempRed << ",0," << (int)tempBlue << ",1);\">";
        html << fixed << setprecision(1) << t_ttt[i] << " °C</span></b></td>";

        // Das anzuzeigende Wettersymbol wird Anhand der Messwerte berechnet.
        // Bei Gewitter und teilweise in der Nacht werden Symbole angezeigt. In diesem Fall wird die Größe etwas reduziert.
        html << "<td rowspan=\"2\"><span style=\"font-size:";
        if (t_wwT[i] > 5 && (t_n[i] <= 75 && (tempUhr < 6 || tempUhr > 22) && (t_r101[i] <= 30 && abs(t_rr1[i]) <= 1))) html << "14";
        // bei nächtlichen Gewittern werden sogar drei Symbole angezeigt, daher noch kleinere Größe.
        else if (t_wwT[i] > 5 || (t_n[i] <= 75 && (tempUhr < 6 || tempUhr > 17) && (t_r101[i] <= 30 && abs(t_rr1[i]) <= 1))) html << "20";
        else html << "26";
        html << "pt; margin-left:42px;\">";

        // Symbolauswahl je nach Bewölkungsgrad. Nachts Mondphasensymbol statt Sonne.
        if ((t_r101[i] < 30 && t_rr1[i] < 1) && t_wwM6[i] <= 10)
            if (t_n[i] <= 30 && (tempUhr < 6 || tempUhr > 22)) html << phase;
            else if (t_n[i] <= 30 && (tempUhr >= 6 || tempUhr <= 22)) html << "☀️";
            else if (t_n[i] <= 60 && (tempUhr < 6 || tempUhr > 22)) html << phase << "☁️";
            else if (t_n[i] <= 60 && (tempUhr >= 6 || tempUhr <= 22)) html << "🌤️";
            else if (t_n[i] <= 75 && (tempUhr < 6 || tempUhr > 22)) html << phase << "☁️";
            else if (t_n[i] <= 75 && (tempUhr >= 6 || tempUhr <= 22)) html << "⛅";
            else html << "☁️";

        // Symbolauswahl bei Regen, Schnee oder Nebel.
        else if (t_n[i] <= 50 && t_wwM6[i] <= 10 && t_RRS1c[i] == 0) html << "🌦️";
        else if (t_wwM6[i] > 10) html << "🌫️";
        else if (t_RRS1c[i] > 0.1) html << "🌨️";
        else if (t_RRS1c[i] <= 0.1 || t_rr1[i] > 1) html << "🌧️";

        // Bei Gewittergefahr kommt ein Blitz dazu.
        if (t_wwT[i] > 5) html << "⚡";

        html << "</span></td></tr>";

        // Angabe Bewölkungsgrad in Prozent mit farbvariablem Hintergrund je nach Wert. Wenig bewölkt = blauer; stark bewölkt = grauer.
        if (tempBluen < 0) tempBluen = 0;
        html << setprecision(0) << "<tr style=\"height:30px;\"><td><span style=\"background-color:rgba(" << t_n[i]*2.55 << "," << t_n[i]*2.55 << "," << int(tempBluen) << ",0.6);\">";
        html << t_n[i] << " % ☁️</span></td></tr>";


    // Angabe Regenwahrscheinlichkeit.
    html << "<tr>";
    html << "<td style=\"color:rgba(0,0," << 40+(t_r101[i]*2) << ");\">" << t_r101[i] << "&nbsp;%&nbsp;";
        if (t_r101[i] < 30) html << "🌂️";
        else html << "☔️";

    // Angabe Sonnenscheinwahrscheinlichkeit, wenn Wahrscheinlichkeit > 10 %.
    if (tempUhr != 1 && t_SunD1[i] > 10)
    {
        html << "</td><td style=\"padding-left:25px; color:rgba(" << t_SunD1[i]*2 << "," << t_SunD1[i]*2 << ",0);\">" << t_SunD1[i] << " % ";

        if (t_SunD1[i] < 60) html << "🌞️ </td></tr>";
        else html << "😎️ </td></tr>";
    }
    else html << "</td></tr>";

    // Angabe absolute Niederschlagsmenge, wenn Menge > 0,5 l/m².
    html << "<tr><td style=\"color:rgba(0,0," << 40+(t_r101[i]*2) << ");\">";
    if (t_rr1[i] > 0.5) html << setprecision(1) << t_rr1[i] << " l/m²";

    // Angabe Windrichtung und -geschwindigkeit.
    if (t_FF[i] <= 8) html << "</td><td style=\"padding-top: 2px;\"><span style=\"border: 1px #aaa; border-style: dotted none dotted none; margin-left:12px;\">";
    else  html << "</td><td><span style=\"color:#c00; border: 1px #aaa; border-style: dotted none dotted none; margin-left:12px;\"> ";
    if (t_DD[i] > 23  && t_DD[i] <=  68) html << "↓";
    if (t_DD[i] > 68  && t_DD[i] <= 113) html << "↙";
    if (t_DD[i] > 113 && t_DD[i] <= 158) html << "←";
    if (t_DD[i] > 158 && t_DD[i] <= 203) html << "↖";
    if (t_DD[i] > 203 && t_DD[i] <= 248) html << "↑";
    if (t_DD[i] > 248 && t_DD[i] <= 293) html << "↗";
    if (t_DD[i] > 293 && t_DD[i] <= 338) html << "→";
    if (t_DD[i] > 338 && t_DD[i] <= 360) html << "↘";
    if (t_DD[i] >= 0  && t_DD[i] <   23) html << "↓";
    html << " " << setprecision(0) << t_FF[i]*3.6 << "&nbsp;km/h";
    html << "</span></td></tr><tr style=\"height:21px;\"><td></td>";

    // Angabe der Windböen, wenn die Brise was steifer wird.
    if (t_FX1[i] > 10) html << "<td><span style=\"color:#a00; margin-left:12px;\">Böen: " << t_FX1[i]*3.6 << "</span></td>";
    html << "</tr>\n</table>";

}

    // Zeige Graphen an. Die Quellenangabe ist Lizenzbestandteil des DWD!
    html << "</div><div align=\"center\" style=\"width:90%;\"><h3>🐸️ Temperaturverlauf [°C]</h3><canvas id=\"canvas\" height=\"400\" width=\"900px\"></canvas><br><br>";
    html << "<h3>🌞️ Sonnenscheindauer [%]</h3><canvas id=\"sonne\" height=\"400\" width=\"900px\"></canvas><br><br>";
    html << "<h3>💧️ Niederschlagswahrscheinlichkeit [%]</h3><canvas id=\"regen\" height=\"400\" width=\"900px\"></canvas><br><br>";
    html << "<h3>🌬️ Windgeschwindigkeit [km/h]</h3><canvas id=\"wind\" height=\"400\" width=\"900px\"></canvas></div><br>Quelle: Deutscher Wetterdienst";

    html.close();
    return 0;
}
