/*   mosmixparser - parses MOSMIX open data from DWD to create a
 *   HTML5 based weather forecast page.
 *
 *   Copyright (C) 2021 Thomas Rudolph, <https://rudo.info/mosmix.php>
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "mosmixparser.h"
#include "mosmixhtml.cpp"

// Schreibe CSV-Datei mit den Haupt-Messwerte.
int csvWrite(char* argv[], uint8_t cPos) {
    ofstream csv(argv[cPos], ios::out);

    if (!csv.is_open())
    {
        cout << "CSV-Datei konnte nicht geschrieben werden.\n";
        exit(0);
    }

    for (uint8_t i=0; i< 246; i++)
    {
        csv << timestamp[i] << "\t" << ttt[i] << "\t" << ttt[i]+e_ttt[i] << "\t" << ttt[i]-e_ttt[i] << "\t" << n[i] << "\t" << rr1[i] << "\t" << r101[i] << "\n";
    }
    csv.close();
    return 0;
}


// Berechne Mondphase.
string moonPhase(uint16_t jahrTag, uint16_t jahr)
{
    float k, l, m, o;
    uint16_t p;

    l = jahrTag*0.002737926;
    m = jahr + l;
    k = (m - 2000) * 12.3685 - 0.19;
    p = k;
    o = k -p;
    o = o + .05;

    string phase;
    if (o < .06 || o > 0.93) phase = "🌑️";
    if (o >= 0.06 && o <= 0.18) phase = "🌒️";
    if (o >= 0.18 && o <= 0.31) phase = "🌓️";
    if (o >= 0.31 && o <= 0.43) phase = "🌔️";
    if (o >= 0.43 && o <= 0.57) phase = "🌕️";
    if (o >= 0.57 && o <= 0.68) phase = "🌖️";
    if (o >= 0.68 && o <= 0.81) phase = "🌗️";
    if (o >= 0.81 && o <= 0.93) phase = "🌘️";

    return phase;
}


// Schreiben der DWD-MOSMIX-Werte einer Zeile und Abspeichern als Array.
float examine(string lines)
{
    if ((lines.find("dwd:value") != string::npos))
    {

        int pos = lines.find_first_of(">");

        for (uint8_t i=0; i<246; i++)
        {
            string temp = lines.substr(pos+4+11*i,10);
            try {
                if (waitTTT) ttt[i] = stof(temp)-273.15;
                if (waitETTT) e_ttt[i] = stof(temp);
                if (waitN) n[i] = stof(temp);
                if (waitRR1) rr1[i] = stof(temp);
                if (waitR101) r101[i] = stof(temp);
                if (waitwwM6) wwM6[i] = stof(temp);
                if (waitRRS1c) RRS1c[i] = stof(temp);
                if (waitFF) FF[i] = stof(temp);
                if (waitDD) DD[i] = stof(temp);
                if (waitFX1) FX1[i] = stof(temp);
                if (waitSunD1) SunD1[i] = stof(temp);
                if (waitwwT) wwT[i] = stof(temp);
            }
            catch( ... )
            {
                cout << "Es wurden ungültige Werte eingelesen. Möglicherweise ist die MOSMIX-Datei\nunvollständig oder für die Station existieren einzelne Werte nicht.\n";
                break;
            }
        }
    }
        waitTTT = 0;
        waitETTT = 0;
        waitN = 0;
        waitRR1 = 0;
        waitR101 = 0;
        waitwwM6 = 0;
        waitRRS1c = 0;
        waitFF = 0;
        waitDD = 0;
        waitFX1 = 0;
        waitSunD1 = 0;
        waitwwT = 0;

        return 0;
}


// Einlesen der DWD-MOSMIX-Datei in die Arrays der jeweiligen Werte.
int parse(char* argv[], uint8_t mPos)
{
    char line[3000];
    uint8_t stampCount = 0;
    string lines;

    ifstream kml (argv[mPos]);
    if (!kml)
    {
        cout << "MOSMIX-Datei nicht gefunden. Ende.\n";
        exit(0);
    }

    while (kml) {
        kml.getline (line, 3000);

        lines = line;

        if ((lines.find("dwd:TimeStep") != string::npos) && (stampCount < 246))
        {
                int pos = lines.find_first_of(">");
                if (lines.length() > pos+23) timestamp[stampCount] = lines.substr(pos+1,23);
                stampCount++;
        }

        // Sofern wir gerade in der Schleife oben eine passende Datenkategorie eingelesen haben, können wir jetzt diesen Datensatz in ein Array schreiben.
        if (waitTTT || waitETTT || waitN || waitRR1 || waitR101 || waitwwM6 || waitRRS1c || waitFF || waitDD || waitFX1 || waitSunD1 || waitwwT)
            if (lines.length() > 2750) examine(lines);

        // Markiere, welche Variable als nächstes kommt, damit examine(lines) die Werte in die richtigen Arrays schreiben kann.
        if ((lines.find("\"TTT\"") != string::npos)) waitTTT = 1;
        if ((lines.find("\"E_TTT\"") != string::npos)) waitETTT = 1;
        if ((lines.find("\"Neff\"") != string::npos)) waitN = 1;
        if ((lines.find("\"RR1c\"") != string::npos)) waitRR1 = 1;
        if ((lines.find("\"R101\"") != string::npos)) waitR101 = 1;
        if ((lines.find("\"wwM\"") != string::npos)) waitwwM6 = 1;
        if ((lines.find("\"RRS1c\"") != string::npos)) waitRRS1c = 1;
        if ((lines.find("\"FF\"") != string::npos)) waitFF = 1;
        if ((lines.find("\"DD\"") != string::npos)) waitDD = 1;
        if ((lines.find("\"FX1\"") != string::npos)) waitFX1 = 1;
        if ((lines.find("\"SunD1\"") != string::npos)) waitSunD1 = 1;
        if ((lines.find("\"wwT\"") != string::npos)) waitwwT = 1;

    }
   return 0;
}

int main(int argc, char* argv[])
{
    if (argc < 4)
    {
        cout << "MOSMIX-Wetterdatenparser 1.0.1 von Thomas Rudolph, https://rudo.info/mosmix.php\n\n";
        cout << argv[0] << " -m input [-c output.html] [-h output.html]\n";
        cout << "  -c  Dateiname für CSV-Ausgabe \n";
        cout << "  -h  Dateiname für HTML-Ausgabe \n";
        cout << "  -m  einzulesende MOSMIX-Datei im kml-Format \n";
        exit(0);
    }

    uint8_t mPos = 0, cPos = 0, hPos = 0;

    for (int i=1;i<argc;i++)
    {
        if (argv[i][0] == '-')
        {
            if (i < argc-1)
            {
                if (argv[i][1] == 'c') cPos = i+1;
                if (argv[i][1] == 'h') hPos = i+1;
                if (argv[i][1] == 'm') mPos = i+1;
            }
        }
    }

    if (mPos > 0) parse(argv, mPos);
    if (cPos > 0) csvWrite(argv, cPos);
    if (hPos > 0) html(argv, hPos);
}
